# coding=utf-8

"""Checkers applied on several types of source files"""

import os
import os.path as osp
from subprocess import Popen, PIPE
import tempfile

from aslint.logger import logger
from aslint.i18n import _
from aslint.config import ASCFG
from aslint.base_checkers import (
    TextMsg, GenericMsg, CheckList, Report,
    checkers_from_context, check_filename,
    DirnameCat,
)


class ErrorMessages(DirnameCat, GenericMsg):

    """Unused or missing messages
    All error messages must be called in a source file and all messages called
    must exist."""
    id = "C9003"
    fmt = "%(id)s: %(label)s"

    def search(self, srcdir):
        """Check encoding of txt"""
        result = []
        # only in 'src'
        if not osp.isdir(osp.join(srcdir, 'bibpyt', 'Messages')):
            return result
        found = False
        subdirs = ['build', 'build/std', 'build/mpi']
        if ASCFG.get('waf.builddir') not in subdirs:
            subdirs.insert(0, ASCFG.get('waf.builddir'))
        for bld in ('release', 'debug'):
            for sub in subdirs:
                instdir = osp.join(srcdir, sub, bld)
                fconf = osp.join(instdir, 'data', 'config.txt')
                if osp.isfile(osp.join(instdir, 'config.txt')) or \
                    osp.isfile(fconf):
                    found = True
                    break
        if not found:
            return [": can not check messages, 'config.txt' not found in "
                    "debug/release of this directories {0}"
                    .format(tuple(subdirs))]
        try:
            unused, not_found = check_messages(instdir, srcdir)
            result.extend([': %s (unused)' % msg for msg in unused])
            result.extend([': %s (missing)' % msg for msg in not_found])
        except ValueError, exc:
            result.append(': check failed: %s' % str(exc))
        return result


class Supv002aRequirement(DirnameCat, GenericMsg):

    """Requirements missed to check spelling of messages (supv002a will fail)
    The testcase supv002a checks the spelling of error messages.
    It requires ``aspell`` and the dictionnaries
    ``$HOME/dev/codeaster/devtools/share/spell/code_aster_dict.aspell.per`` and
    ``code_aster_cata.aspell.per``.
    """
    id = "W9004"
    fmt = "%(id)s: %(label)s"

    def search(self, srcdir):
        """Check pre-requisites for supv002a"""
        share = osp.join(ASCFG.get('devtools_root'), 'share', 'spell')
        fdict = osp.join(share, 'code_aster_dict.aspell.per')
        if not osp.isfile(fdict):
            return [": french dictionnary is missing: %s" % fdict]
        fdict = osp.join(share, 'code_aster_cata.aspell.per')
        if not osp.isfile(fdict):
            return [": dictionnary of keywords is missing: %s (see help)"
                    % fdict]
        try:
            Popen(['aspell', '--version'], stdout=PIPE).wait()
        except OSError:
            return [": 'aspell' not found, not in $PATH"]
        return []


CHECK_LIST = checkers_from_context(globals(), TextMsg)


def check_messages(instdir, srcdir):
    """Check the errors messages"""
    from aslint.messages import MessagesManager, CataMessageError
    tmpcache = tempfile.NamedTemporaryFile(prefix='cachedict.').name
    catapy = osp.abspath(osp.join(srcdir, 'code_aster'))
    if not osp.exists(catapy):
        catapy = osp.abspath(osp.join(srcdir, 'catapy'))
    msgman = MessagesManager(repref=instdir,
                             fort=osp.abspath(osp.join(srcdir, 'bibfor')),
                             pyt=osp.abspath(osp.join(srcdir, 'bibpyt')),
                             capy=catapy,
                             cache_dict=tmpcache,
                             verbose=False, force=True)
    msgman.read_cata()
    try:
        unused, not_found = msgman.check_cata()
    except CataMessageError, msg:
        logger.error(_("error during check of error messages:\n%s"), str(msg))
        raise ValueError
    finally:
        try:
            os.remove(tmpcache)
        except OSError:
            pass
    return unused, not_found


def check_global(srcdir):
    """Check a directory. Return a report of the errors."""
    checklist = CheckList()
    checklist.register(CHECK_LIST)
    report = Report()
    logger.info(_("add checking of error messages"))
    lmsg = check_filename(srcdir, checklist.on_dirname())
    report.set(srcdir, lmsg)
    return report
